
require 'singleton'

module Flow
  class Config
    include Singleton

    def initialize
      @config = self.defaults
    end

    def defaults
      {
        url: 'https://flow.ribbon.co'.freeze,
        api_key: nil
      }
    end

    def method_missing(meth, *args, &block)
      meth_str = meth.to_s

      if /^(\w+)=$/.match(meth_str)
        _set($1, args.first)
      else
        _get(meth)
      end
    end

    def url=(url)
      raise Flow::Errors::ConfigError, "URL must be defined" unless url
      url = url[0..-2] if url[-1] == '/' # No trailing forward-slash
      _set(:url, url)
    end

    def to_s
      return @config.to_s
    end

    private

    def _set(key, value)
      # Changing the config value should require calling the
      # _set method again.
      return @config[key.to_sym] = value.nil? ? nil : value.dup.freeze
    end

    def _get(key)
      return @config[key.to_sym]
    end
  end
end