
module Flow
  module Models

    class Model
      attr_reader :model_hash
      attr_reader :connection

      def initialize(connection, model_hash)
        @connection = connection
        @model_hash = model_hash
      end

      def reload
        self.class.load(connection, self.token)
      end

      def reload!
        updated_version = self.reload
        @model_hash = updated_version.model_hash
        self
      end

      def method_missing(meth, *args, &block)
        if model_hash.key?(meth.to_s)
          return Model.load_model(meth, connection, model_hash[meth.to_s])
        end

        super
      end

      ############################################################
      # Class methods
      ############################################################
      class << self
        def __load_path
          # Really wish I had Rail's pluralize here, but this
          # fits all of our cases for now and should fit most
          # if not all of our cases going forward.
          "#{__model_name}s"
        end

        def __model_name
          model_name = self.name.split('::').last.downcase
          raise "should not call this for abstract model" if model_name == 'model'
          model_name
        end

        def build(connection, model_hash)
          # Default behavior is just to create an instance
          # of the class.
          self.new(connection, model_hash)
        end

        def load(connection, token)
          r = connection.get("#{__load_path}/#{token}")
          raise Flow::Errors::LoadError.new(r.message) unless r.success?

          return r.send(__model_name)
        end

        def load_model(name, connection, model_hash)
          name = name.to_sym

          case name
          when :pool
            return Flow::Models::Pool.build(connection, model_hash)
          when :card
            return Flow::Models::Card.build(connection, model_hash)
          when :transaction
            return Flow::Models::Transaction.build(connection, model_hash)
          else
            return model_hash
          end
        end
      end # class << self

    end # Model
  end # Models
end # Flow