
require 'json'

module Flow

  class Response
    attr_reader :body
    attr_reader :connection

    def initialize(connection, rest_client_response)
      @connection = connection
      @response = rest_client_response

      begin
        body = @response.body
        @body = JSON.parse(body) unless body.empty?
        @body ||= {}
        _deep_freeze(@body)
      rescue JSON::ParserError => e
        raise Flow::Errors::ResponseError, "Invalid JSON returned: #{e.message}"
      end
    end

    def http_success?
      @response.code >= 200 && @response.code < 300
    end

    def success?
      body['status'] == 'success'
    end

    def error?
      body['status'] == 'error'
    end

    def method_missing(meth, *args, &block)
      if body.key?(meth.to_s)
        return Flow::Models::Model.load_model(meth, connection, body[meth.to_s])
      end

      super
    end


    private

    def _deep_freeze(obj)
      if obj.respond_to?(:each)
        obj.each do |elem|
          _deep_freeze(elem)
        end
      end

      obj.freeze
    end
  end

end